package sensor.test.app.reconnectingapp;

import android.util.Log;

import com.mbientlab.metawear.Data;
import com.mbientlab.metawear.MetaWearBoard;
import com.mbientlab.metawear.Route;
import com.mbientlab.metawear.Subscriber;
import com.mbientlab.metawear.builder.RouteBuilder;
import com.mbientlab.metawear.builder.RouteComponent;
import com.mbientlab.metawear.module.AmbientLightLtr329;

import java.util.concurrent.TimeUnit;

import bolts.Task;

public class LightSensor {
    private AmbientLightLtr329 module;

    public LightSensor(final MetaWearBoard mwBoard) {
        module = mwBoard.getModule(AmbientLightLtr329.class);
    }

    public void configure() {
        // Set the gain to 8x
        // Set integration time to 250ms
        // Set measurement rate to 2s
        module.configure()
                .gain(AmbientLightLtr329.Gain.LTR329_8X)
                .integrationTime(AmbientLightLtr329.IntegrationTime.LTR329_TIME_250MS)
                .measurementRate(AmbientLightLtr329.MeasurementRate.LTR329_RATE_2000MS)
                .commit();
    }

    public void start() {
        Log.i("LIGHT","Starting ambient light sensor");
        module.illuminance().start();
    }

    public boolean addRoute() {
        return addRoute(module.illuminance().addRouteAsync(new RouteBuilder() {
            @Override
            public void configure(RouteComponent source) {
                Log.i("LIGHT","Light data Route setup");
                source.stream(new Subscriber() {
                    @Override
                    public void apply(Data data, Object... env) {
                        Log.i("LIGHT","Illuminance == " + data.value(Float.class));
                    }
                });
            }
        }), "Light", 10000);
    }

    // In our app we call stop when we "cleanly" exit application
    // When bluetooth is switched off or unexpectedly disconnected
    // this is not called
    // Question is - what happens after tearDown() on next connectAsync() - I do configure, addRouteAsync(),
    // start()
    // do I need to stop() my sensor modules, or how to handle this?
    public void stop() {
        module.illuminance().stop();
    }

    protected boolean addRoute(final Task<Route> rTask, final String module, final long millis) {
        try {
            rTask.waitForCompletion(millis, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            Log.e(module, "addRoute() for " + module + " interrupted." + e);
            Log.e(module,"cause " + e.getCause());
            return false;
        }

        if (rTask.isFaulted() || rTask.isCancelled()) {
            Log.e(module, "data Route was not set: " + rTask.getError());
            if (rTask.getError() != null) {
                for (StackTraceElement ste : rTask.getError().getStackTrace()) {
                    Log.e(module,"-- " + ste);
                }
            }
            return false;
        }

        return true;
    }
}
