package sensor.test.app.reconnectingapp;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothManager;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanResult;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.RequiresApi;
import android.support.design.widget.BottomNavigationView;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.MenuItem;
import android.widget.TextView;

import java.util.List;

public class MainActivity extends AppCompatActivity {

    private final String macAddress = "F2:66:56:BC:EB:6E";

    private TextView mTextMessage;
    private MetawearDevice device;

    private BottomNavigationView.OnNavigationItemSelectedListener mOnNavigationItemSelectedListener
            = new BottomNavigationView.OnNavigationItemSelectedListener() {

        @Override
        public boolean onNavigationItemSelected(@NonNull MenuItem item) {
            switch (item.getItemId()) {
                case R.id.navigation_home:
                    mTextMessage.setText(R.string.title_home);
                    return true;
                case R.id.navigation_dashboard:
                    mTextMessage.setText(R.string.title_dashboard);
                    return true;
                case R.id.navigation_notifications:
                    mTextMessage.setText(R.string.title_notifications);
                    return true;
            }
            return false;
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        device = null;

        setContentView(R.layout.activity_main);

        mTextMessage = (TextView) findViewById(R.id.message);
        BottomNavigationView navigation = (BottomNavigationView) findViewById(R.id.navigation);
        navigation.setOnNavigationItemSelectedListener(mOnNavigationItemSelectedListener);

        // Listen on bluetooth adapter availability
        this.getApplicationContext().registerReceiver(mReceiver,
                new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED));

        BluetoothManager bm = (BluetoothManager) getApplicationContext()
                                        .getSystemService(Context.BLUETOOTH_SERVICE);
        Log.i("MainActivity", "onCreate()");
        if (BluetoothAdapter.STATE_ON == bm.getAdapter().getState()) {
            Log.i("MainActivity", "onCreate() - BLE is ON, starting");
            startDevice();
        }
    }

    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
        public void onReceive (final Context context, final Intent intent) {
            final String action = intent.getAction();

            if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
                final int extraState =
                        intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, -1);
                switch(extraState) {
                    case BluetoothAdapter.STATE_OFF:
                        Log.i("MAIN", "Switching sensor off. Bluetooth turning off.");
                        destroyDevice();
                        break;
                    case BluetoothAdapter.STATE_ON:
                        Log.i("MAIN", "Bluetooth was switched on.");
                        startDevice();
                        break;
                    default:
                }
            }
        }
    };


    private synchronized void startDevice() {
        Log.i("MAIN", "Starting device connection.");
        if (device == null) {
            device = new MetawearDevice(getApplicationContext(),
                    macAddress);

            device.bindBluetoothService();
        }
    }

    private synchronized void destroyDevice() {
        Log.i("MAIN", "Dropping device");
        if (device != null) {
            device.unbindBluetoothService();
            device.destroy();
            device = null;
        }
    }
}
